<?php

/**
 * Zlick Payments Plugin
 *
 * @category  Zlick
 * @package   Zlick
 * @author    Arsalan Ahmad <arsalan@zlick.it>
 * @copyright Copyright (c) 2018 Zlick ltd (https://www.zlick.it)
 * @license   http://opensource.org/licenses/osl-3.0.php  Open Software License (OSL 3.0)
 * @link      https://www.zlick.it
 */

function zlickpay_widget_placeholder()
{
	return '';
}

/**
 * Returns the smart prc widget.
 *
 * @param string $post_id The post id.
 *
 * @return string
 */
function zlickpay_widget_link_shortcode($data)
{

	if (!is_single()) {
		return '';
	}

	$post_id = get_the_ID();
	if (!empty($post_id)) {
		$post_title = get_the_title($post_id);
		$zp_custom_fields = get_post_custom($post_id);
		$article_price = '';
		if (zlickpay_cookie_is_valid_subscription() or zlickpay_cookie_is_purchased_article(trim($post_id))) {
			return '';
		} else {
			$widget_text = get_option('zp_widget_text');
			$zp_enable_subscription = get_option('zp_enable_subscription', '');
			$zp_article_price = zlickpay_sanitize_price($article_price);
			$zp_widget_text = str_replace('{price}', $article_price, $widget_text);
			// Documented in readme under "Technical" section.
			wp_enqueue_style('zlick-payments-fade-css', esc_url(plugins_url('styles/fade.css', dirname(__FILE__))), false, '1.0', 'all');
			wp_enqueue_script('zlick-payments-fade-js', esc_url(plugins_url('js/fade.js', dirname(__FILE__))), '', '', true);
			wp_enqueue_script('zlick-payments-sdk', 'https://cdn.zlick.it/zlick-paywall-element-2.0.0.js', '', '', true);
			wp_enqueue_script(ZLICK_PLUGIN_NAME, array('jquery', 'zlick-payments-sdk'), '1.0', true);
			wp_localize_script(
				ZLICK_PLUGIN_NAME,
				'zlick_payments_ajax',
				array(
					'ajax_url' => ZLICK_AJAX_URL,
					'client_token' => zlickpay_get_options_data('zp_client_token'),
					'user_id' => zlickpay_get_current_user_id(),
					'article_id' => $post_id,
					'zp_article_name' => $post_title,
					'zp_plugin_version' => ZLICK_PLUGIN_VERSION,
					'zp_ajax_nonce' => wp_create_nonce("zp-ajax-nonce")
				)
			);
			ob_start();
			include ZLICK_PLUGIN_DIR . 'public/templates/zlick-widget.php';
			$widget = ob_get_contents();
			ob_end_clean();

			return $widget;
		}
	}

	return '';
}

/**
 * Register short codes.
 */
add_shortcode('zlick_payment_widget', 'zlickpay_widget_link_shortcode');
add_shortcode('zp_placeholder', 'zlickpay_widget_placeholder');

/**
 * Register Ajax callback for logged in and anonymous users
 */
add_action('wp_ajax_zp_authenticate_article', 'zlickpay_authenticate_article');
add_action('wp_ajax_nopriv_zp_authenticate_article', 'zlickpay_authenticate_article');

function zlickpay_shortcode_resource()
{
	wp_register_script(ZLICK_PLUGIN_NAME, plugins_url("js/zlick-widget.js", __DIR__), array('jquery'), "1.0", false);
}

add_action('init', 'zlickpay_shortcode_resource');

if (!function_exists('zlickpay_cookie_register_article')) {

	function zlickpay_cookie_register_article($article_id)
	{

		$secret = zlickpay_get_options_data('zp_client_secret');
		// $zp_article_cookie_content signed token
		$zp_article_cookie_content = sanitize_text_field(isset($_COOKIE["zp_articles"]) ? $_COOKIE["zp_articles"] : '');

		if (empty($zp_article_cookie_content) || !isset($zp_article_cookie_content)) {
			$authed_articles = array();
		} else {
			$payload = zlickpay_get_jwt_payload($zp_article_cookie_content);
			$authed_articles = explode(",", $payload);
		}


		$authed_articles[] = $article_id;
		$authed_articles = array_unique($authed_articles);

		$payload = join(",", $authed_articles);

		setcookie("zp_articles", zlickpay_jwt_sign($secret, $payload), time() + 60 * 60 * 24 * 365 * 10, "/");
	}
}

if (!function_exists('zlickpay_cookie_register_subscription')) {
	function zlickpay_cookie_register_subscription()
	{
		$zp_options = zlickpay_get_options_data();
		$client_token = $zp_options['zp_client_token'];
		$client_secret = $zp_options['zp_client_secret'];

		$cookie_val = zlickpay_jwt_sign($client_secret, $client_token);

		setcookie("zp_subscription", $cookie_val, time() + 60 * 60 * 3, "/");
	}
}

if (!function_exists('zlickpay_cookie_unregister_all')) {
	function zlickpay_cookie_unregister_all()
	{
		setcookie("zp_subscription", "",  time() - 10000, "/");
		setcookie("zp_articles", "",  time() - 10000, "/");
	}
}

if (!function_exists('zlickpay_authenticate_article')) {

	/**
	 * @return false|int|null
	 */
	function zlickpay_authenticate_article()
	{

		// Check for nonce security
		if (!isset($_POST['zp_nonce']) || !wp_verify_nonce($_POST['zp_nonce'], 'zp-ajax-nonce')) {
			// Replying in code so that we don't reveal exactly what went wrong to the attacker
			// But we can still identify causes of the issues
			wp_die('Request could not be verified: Code ZPZW101');
		}

		$zp_options = zlickpay_get_options_data();
		$client_token = $zp_options['zp_client_token'];
		$client_secret = $zp_options['zp_client_secret'];
		$zp_signed = sanitize_text_field($_POST['zp_signed']);

		if (!zlickpay_validate_sign($client_secret, $zp_signed)) {
			wp_die('Request could not be verified: Code ZPZW102');
		}

		$signedPayload = json_decode(zlickpay_get_jwt_payload($zp_signed), true);

		if (isset($signedPayload["transaction"]["hasAccess"]) && $signedPayload["transaction"]["hasAccess"] === true) {
			$article_id = intval($signedPayload["transaction"]["productId"]);
			if (!$article_id) {
				wp_die('Request could not be verified: Code ZPZWA101');
			}
			zlickpay_cookie_register_article($article_id);
		} elseif (isset($signedPayload["subscription"]["hasAccess"]) && $signedPayload["subscription"]["hasAccess"] === true) {
			zlickpay_cookie_register_subscription();
		} else {
			zlickpay_cookie_unregister_all();
		}

		wp_die("reload");

		return 'Done. Reload the page now';
	}
}

if (!function_exists('zlickpay_cookie_is_valid_subscription')) {

	function zlickpay_cookie_is_valid_subscription()
	{
		// $cookie_val contains jwt signed string that is validated before further use
		$cookie_val = sanitize_text_field(isset($_COOKIE["zp_subscription"]) ? $_COOKIE["zp_subscription"] : "");
		$secret = zlickpay_get_options_data('zp_client_secret');

		if ($cookie_val === "") {
			return false;
		}
		if (!zlickpay_validate_sign($secret, $cookie_val)) {
			return false;
		}

		$payload = zlickpay_get_jwt_payload($cookie_val);

		if ($payload === zlickpay_get_options_data('zp_client_token')) {
			return true;
		}
		return false;
	}
}

if (!function_exists('zlickpay_cookie_is_purchased_article')) {

	function zlickpay_cookie_is_purchased_article($article_id)
	{
		// $cookie_val contains jwt signed string that is validated before further use
		$cookie_val = sanitize_text_field(isset($_COOKIE["zp_articles"]) ? $_COOKIE["zp_articles"] : "");
		$secret = zlickpay_get_options_data('zp_client_secret');

		if ($cookie_val === "") {
			return false;
		}
		if (!zlickpay_validate_sign($secret, $cookie_val)) {
			return false;
		}

		$payload = zlickpay_get_jwt_payload($cookie_val);
		$purchased_articles = explode(",", $payload);

		return in_array($article_id, $purchased_articles);
	}
}

if (!function_exists('zlickpay_find_number_paragraphs')) {
	/**
	 * Calculate previewable paragrpahs
	 *
	 * @param $content
	 * @param $para_count
	 * @param $is_block_editor
	 *
	 * @return false|string
	 */
	function zlickpay_find_number_paragraphs($content, $para_count, $is_block_editor)
	{
		$para_i = 0;
		$char_count = 0;
		$needle = $is_block_editor ? '<!-- /wp:paragraph -->' : '</p>';

		do {
			$para_i++;
			$offset = $char_count + 1;
			if ($offset >= strlen($content)) {
				break;
			}
			$ti = strpos($content, $needle, $offset);
			if ($ti === FALSE) {
				break;
			}
			$char_count = $ti;
		} while ($para_i < $para_count);

		return $char_count;
	}
}

function zlickpay_add_noarchive_meta()
{
	$post_id = get_the_ID();
	$zp_custom_fields = get_post_custom($post_id);

	if (!isset($zp_custom_fields['zp_is_paid']) || $zp_custom_fields['zp_is_paid'][0] != 'paid') {
		return;
	}
	// Now it means that the article is paid so it should not be archived
	echo '<meta name="robots" content="noarchive">';
	return;
}

if (!function_exists('zlickpay_limit_content')) {
	/**
	 * Limit Content renerding to previewable_paragraph option
	 *
	 * @param $content
	 *
	 * @return false|string
	 */
	function zlickpay_limit_content($content)
	{

		$post_id = get_the_ID();
		$zp_custom_fields = get_post_custom($post_id);

		if (!isset($zp_custom_fields['zp_is_paid']) || $zp_custom_fields['zp_is_paid'][0] != 'paid') {
			return $content;
		}

		if (empty(zlickpay_get_options_data('zp_client_token'))) {
			echo '<!-- zp client token has not been set -->';
			return $content;
		}

		if (zlickpay_get_options_data('zp_search_engine_bypass') && zlickpay_is_search_engine()) {
			return $content;
		}

		// gutenberg editor gives content in paragraphs with many newlines. No <p> tags.
		// classic editor gives a mix of html and new line seperated paragraphs

		$is_block_editor = false;

		if (strpos($content, "<!-- /wp:paragraph -->") > 0) {
			$is_block_editor = true;
		}

		if (!$is_block_editor) {
			$content = wpautop($content);
			remove_filter('the_content', 'wpautop');
		}

		$needle = "<!--zlick-paywall-->";
		$content_length = strpos($content, $needle);

		if (get_post_type() == ZLICK_POST_TYPE_POST) {
			if (!zlickpay_cookie_is_valid_subscription() && !zlickpay_cookie_is_purchased_article($post_id)) {
				if ($content_length === false) {
					$para_count = zlickpay_get_options_data('zp_previewable_paras');

					$content_length = zlickpay_find_number_paragraphs($content, $para_count, $is_block_editor);
				}
				$content = substr($content, 0, $content_length);
				$content .= do_shortcode("[zlick_payment_widget post_id=" . $post_id . "]");
			}
		}

		return $content;
	}
}

add_filter("the_content", "zlickpay_limit_content", -1);
add_action('wp_head', 'zlickpay_add_noarchive_meta');

if (!function_exists('zlickpay_limit_feed_content')) {
	/**
	 * Limit Content renerding to previewable_paragraph option
	 *
	 * @param $content
	 * @since 2.7.4
	 * @return false|string
	 */
	function zlickpay_limit_feed_content($content)
	{

		$post_id = get_the_ID();
		$zp_custom_fields = get_post_custom($post_id);

		if (!isset($zp_custom_fields['zp_is_paid']) || $zp_custom_fields['zp_is_paid'][0] != 'paid') {
			return $content;
		}

		if (empty(zlickpay_get_options_data('zp_client_token'))) {
			echo '<!-- zp client token has not been set -->';
			return $content;
		}

		// gutenberg editor gives content in paragraphs with many newlines. No <p> tags.
		// classic editor gives a mix of html and new line seperated paragraphs

		$is_block_editor = false;

		if (strpos($content, "<!-- /wp:paragraph -->") > 0) {
			$is_block_editor = true;
		}

		if (!$is_block_editor) {
			$content = wpautop($content);
			remove_filter('the_content', 'wpautop');
		}

		$needle = "<!--zlick-paywall-->";
		$content_length = strpos($content, $needle);

		if (get_post_type() == ZLICK_POST_TYPE_POST) {
			if ($content_length === false) {
				$para_count = zlickpay_get_options_data('zp_previewable_paras');

				$content_length = zlickpay_find_number_paragraphs($content, $para_count, $is_block_editor);
			}
			$content = substr($content, 0, $content_length);
		}

		return $content;
	}
}

add_filter( 'the_content_feed', "zlickpay_limit_feed_content");


if (!function_exists('zlickpay_get_options_data')) {
	/**
	 * Retrieve saved configurations option by key|all
	 *
	 * @return array
	 */
	function zlickpay_get_options_data($key = 'all')
	{
		$zlick_options = get_option('zlick_payments_settings', []);

		if (empty($zlick_options)) {
			return;
		}

		return $key == 'all' ? $zlick_options : $zlick_options[$key];
	}
}

if (!function_exists('zlickpay_sanitize_price')) {
	/**
	 * Round the Price upto 2 decimal values
	 *
	 * @return array
	 */
	function zlickpay_sanitize_price($price)
	{
		$iprice = (float) $price;
		$iprice = round($iprice * 100);
		return $iprice;
	}
}
